"
COPYRIGHT (c) 2021 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGBehaviorStrategyUser subclass:#RGBehavior
	instanceVariableNames:'superclass localMethods traitComposition metaclass tagsForMethods
		tags'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGBehavior comment:'An RGBehavior is an abstract definition for class-alike entities (e.g. classes, traits)

Instance Variables
	methods:		<Collection>
	protocols:		<Collection>
	superclass:		<Object>'
!

!RGBehavior class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2021 LabWare


"
! !

!RGBehavior class methodsFor:'instance creation'!

newClass

    ^ self unnamedWithBehaviorStrategy: RGClassStrategy basicNew
!

newMetaclass

    ^ self unnamedWithBehaviorStrategy: RGMetaclassStrategy basicNew
!

newMetaclassTrait

    ^ self unnamedWithBehaviorStrategy: RGMetaclassTrait newStrategy
!

newTrait

    ^ self unnamedWithBehaviorStrategy: RGTrait newStrategy
! !

!RGBehavior class methodsFor:'as yet unclassified'!

badInstantiationError

    RGBadInstantiationError signal: 'I cannot be instantiated this way because every RGBehavior must know its proper behavior strategy. Please use a subclass of RGBehaviorFactory or my methods like #newClass'
!

named: aString

    ^ self badInstantiationError
!

named: aName behaviorStrategy: anRGBehaviorStrategy

    | aBehavior |
    
    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        initialize.
    anRGBehaviorStrategy initialize.
    ^ aBehavior
        pvtName: aName asSymbol;
        yourself		
!

named: aString parent: anRGObject

    ^ self badInstantiationError
!

named: aName parent: anRGObject behaviorStrategy: anRGBehaviorStrategy
    
    | aBehavior |

    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        parent: anRGObject;
        initialize.
    anRGBehaviorStrategy initialize.
    ^ aBehavior
        pvtName: aName asSymbol;
        yourself		
!

parent: anRGObject behaviorStrategy: anRGBehaviorStrategy
    
    | aBehavior |

    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        parent: anRGObject;
        initialize.
    anRGBehaviorStrategy initialize.
    ^ aBehavior
!

unnamedWithBehaviorStrategy: anRGBehaviorStrategy

    | aBehavior |
    
    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        initialize.
    anRGBehaviorStrategy initialize.
    ^ aBehavior
!

unresolvedNamed: aName withParent: anRGObject behaviorStrategy: anRGBehaviorStrategy
    
    | aBehavior |

    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        parent: anRGObject;
        initializeUnresolved.
    anRGBehaviorStrategy initializeUnresolved.
    ^ aBehavior
        pvtName: aName asSymbol;
        yourself		
!

unresolvedWithBehaviorStrategy: anRGBehaviorStrategy

    | aBehavior |
    
    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        initializeUnresolved.
    anRGBehaviorStrategy initializeUnresolved.
    ^ aBehavior
!

unresolvedWithParent: anRGObject behaviorStrategy: anRGBehaviorStrategy
    
    | aBehavior |

    aBehavior := self basicNew
        behaviorStrategy: anRGBehaviorStrategy;
        parent: anRGObject;
        initializeUnresolved.
    anRGBehaviorStrategy initializeUnresolved.
    ^ aBehavior
! !

!RGBehavior methodsFor:'accessing'!

compiler

    ^ self class compiler
!

users
    ^ #()
! !

!RGBehavior methodsFor:'accessing - backend'!

addLocalMethod: anRGMethod

    self backend forBehavior addLocalMethod: anRGMethod to: self.
    
    self announcer methodAdded: anRGMethod.
!

addMethodTag: aSymbol 

    self announceDefinitionChangeDuring: [ 
        self backend forBehavior addMethodTag: aSymbol to: self. ].
!

addProtocol: aSymbol 

    self addMethodTag: aSymbol
!

cleanLocalMethods

    | oldMethods |
    
    oldMethods := self localMethods.

    self backend forBehavior cleanLocalMethodsFor: self.

    self cleanProtocols.
    
    oldMethods do: [ :each |
        self announcer methodRemoved: each ]
!

cleanProtocols
    
    self cleanTagsForMethods
!

cleanTags

    self announceDefinitionChangeDuring: [ 
        self cleanTagsWithoutAnnouncemnt ].
!

cleanTagsForMethods

    self backend forPackage cleanTagsForMethodsFor: self
!

cleanTagsWithoutAnnouncemnt

    self backend forBehavior cleanClassTagsFor: self.
!

localMethodsDo: aBlock

    self backend forBehavior localMethodsFor: self do: aBlock
!

metaclass

    ^ self backend forBehavior metaclassFor: self
!

metaclass: anRGMetaclass

    self backend forBehavior setMetaclassFor: self to: anRGMetaclass.
    
!

name: aString

    | oldName subclassesWithOldDefinitions |
    
    oldName := self name.
    
    subclassesWithOldDefinitions := (self subclasses reject: [:each | each isMeta]) collect: [ :each | 
        each -> each copyForBehaviorDefinition ].
    
    self announceDefinitionChangeDuring: [ 
        super name: aString ].
    
    self announcer behaviorRenamed: self from: oldName.

    subclassesWithOldDefinitions do: [ :assoc |
        self announcer behaviorDefinitionChangedFrom: assoc value to: assoc key.
        self announcer behaviorModificationAppliedTo: assoc key.
        self announcer behaviorParentRenamed: assoc key from: oldName ].
!

removeLocalMethod: anRGMethod 

    (anRGMethod package = self package)
        ifFalse: [ anRGMethod package removeExtensionMethod:  anRGMethod ].
        
    self backend forBehavior removeLocalMethod: anRGMethod from: self.
    
    self announcer methodRemoved: anRGMethod.
!

removeMethodTag: aSymbol

    self backend forPackage removeMethodTag: aSymbol from: self.
    
    self localMethodsDo: [ :method | 
        method untagFrom: aSymbol ].
!

removeProtocol: aSymbol

    self removeMethodTag: aSymbol.
!

superclass

    ^ self backend forBehavior superclassFor: self
!

superclass: anRGBehavior

    self announceDefinitionChangeDuring: [ 
        self backend forBehavior setSuperclassFor: self to: anRGBehavior ].
!

tagWith: aSymbol 

    self announceDefinitionChangeDuring: [ 
        self backend forBehavior tagClass: self with: aSymbol.
        self package addClassTag: aSymbol.
    ].
    
!

tagsDo: aBlock

    self backend forBehavior tagsForClass: self do: aBlock
!

tagsForMethodsDo: aBlock

    self backend forBehavior tagsForMethodsFor: self do: aBlock
!

traitComposition

    ^ self backend forBehavior traitCompositionFor: self
!

traitComposition: anRGTraitComposition

    self announceDefinitionChangeDuring: [ 
        self backend forBehavior setTraitCompositionFor: self to: anRGTraitComposition.].
!

unresolveName

    | oldName subclassesWithOldDefinitions |
    
    oldName := self name.
    
    subclassesWithOldDefinitions := (self subclasses reject: [:each | each isMeta]) collect: [ :each | 
        each -> each copyForBehaviorDefinition ].
    
    self announceDefinitionChangeDuring: [ 
        super unresolveName ].
    
    self announcer behaviorRenamed: self from: oldName.

    subclassesWithOldDefinitions do: [ :assoc |
        self announcer behaviorDefinitionChangedFrom: assoc value to: assoc key.
        self announcer behaviorModificationAppliedTo: assoc key.
        self announcer behaviorParentRenamed: assoc key from: oldName ].
!

unresolveSuperclass

    self announceDefinitionChangeDuring: [ 
        self pvtSuperclass: (RGUnresolvedValue recursive) ].
!

untagFrom: aSymbol

    self backend forPackage untagClass: self from: aSymbol
! !

!RGBehavior methodsFor:'accessing - definition'!

classVariablesBindings

    ^ self propertyNamed: #classVariablesBindings ifAbsentPut: [ IdentityDictionary new.]
!

classVariablesString
    "Answer a string of my class variable names separated by spaces."

    ^ self behaviorStrategy classVariablesString
!

sharedPoolsString
    "Answer a string of my class variable names separated by spaces."

    "TODO: check validity"
    ^String streamContents: [ :stream | 
        self sharedPools 
            do: [ :each | stream nextPutAll: each name]
            separatedBy: [ stream space ] ]
!

traitCompositionString
    ^ self traitComposition traitCompositionString
        
! !

!RGBehavior methodsFor:'accessing - model'!

tags

    ^ self tagsSet asArray 
!

tagsForAllMethods
    "I act as #tagsForMethods but I also takes into account methods comming from traits"

    ^ self methods flatCollect: [ :each  | each tags ] as: Set
!

tagsForMethods
    "Retrieves the traits defined in the receiver"

    | allTags |
    allTags := self defaultTagsForMethods.
    self tagsForMethodsDo: [ :each | allTags add: each].
    ^ allTags asArray
!

tagsForMethodsSet
    "Retrieves the traits defined in the receiver"

    | allTags |
    allTags := self defaultTagsForMethods.
    self tagsForMethodsDo: [ :each | allTags add: each].
    ^ allTags
!

tagsSet

    | allTags |
    allTags := self defaultTags.
    self tagsDo: [ :each | allTags add: each].
    ^ allTags 
! !


!RGBehavior methodsFor:'accessing class hierarchy'!

subclasses

    | result |
    result := IdentitySet new.
    self environment behaviorsDo: [ :each |
        (each superclass == self) ifTrue: [ result add: each] ].
    ^ result asArray
! !

!RGBehavior methodsFor:'announcements'!

announceDefinitionChangeDuring: aBlock

    | oldVersion |

    self announcer isSuspended ifTrue: [ 
        aBlock value. 
        ^ self ]	.
        
    self announcer  suspendAllWhile: [ 
        oldVersion := self copyForBehaviorDefinition.].
    aBlock value.
    self announcer 	behaviorDefinitionChangedFrom: oldVersion to: self.
    self announcer behaviorModificationAppliedTo: self.

    
    
! !

!RGBehavior methodsFor:'asYetUnclassified'!

asYetUnclassifiedProtocolName

    ^ RGObject asYetUnclassifiedProtocolName

    "Modified: / 04-02-2021 / 10:09:01 / Jan Vrany <jan.vrany@labware.com>"
!

usedTraits

    ^ self traitComposition usedTraits
! !

!RGBehavior methodsFor:'compiling'!

compile: aString classified: aCategory
    ^ self compile: aString classified: aCategory notifying: nil

    "Created: / 04-02-2021 / 10:07:35 / Jan Vrany <jan.vrany@labware.com>"
!

compile: aString classified: aCategory notifying: anObject

        | parseTree selector newMethod | 
        
        parseTree := (RBParser 
                parseMethod: aString
                onError: [ :msg :pos | ^ nil ])" methodClass: self".
        
        selector := parseTree selector.
        newMethod := self ensureLocalMethodNamed: selector.
        newMethod sourceCode: aString.
        aCategory notNil ifTrue: [ 
                newMethod tagWith: aCategory ].
        
        ^ selector.

    "Created: / 04-02-2021 / 10:06:43 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 15-03-2021 / 15:16:04 / Jan Vrany <jan.vrany@labware.com>"
!

compile: aString notifying: anObject
    ^ self compile: aString classified: self asYetUnclassifiedProtocolName notifying: anObject

    "Created: / 04-02-2021 / 10:18:20 / Jan Vrany <jan.vrany@labware.com>"
! !

!RGBehavior methodsFor:'conversion'!

convertToMetaclassTrait

    | metaclassTraitStrategy originalName |
    
    (self resolvedProperties includesAnyOf: #(traitComposition))
        ifTrue: [self error: 'This class cannot be converted to trait'].
        
    "TODO: check empty layout"
"	newMetaclassTrait := RGMetaclassTrait unresolvedWithParent: self parent.
    newMetaclassTrait copyPropertiesFrom: self.
"
    metaclassTraitStrategy := RGMetaclassTrait newStrategyFor: self environment.
    metaclassTraitStrategy owner: self.
    metaclassTraitStrategy pvtBaseTrait: self behaviorStrategy pvtBaseClass.

    self behaviorStrategy: metaclassTraitStrategy.

    originalName := self name.
    
    (self hasResolvedName)
        ifTrue: [ self pvtName: ((self name withoutSuffix: ' class'), ' classTrait') asSymbol ].
        
    self environment ask replaceName: originalName with: self name.

    
    
    
!

convertToTrait

    | traitStrategy originalName |
    
    (self resolvedProperties includesAnyOf: #("superclass" traitComposition classVariables sharedPools))
        ifTrue: [self error: 'This class cannot be converted to trait'].
        
    "TODO: check empty layout"
    "newTrait := RGTrait unresolvedWithParent: self parent.
    newTrait copyPropertiesFrom: self.
    newTrait pvtName: name.
    newTrait pvtLocalMethods: localMethods.
    newTrait pvtTags: tags.

    newTrait behaviorStrategy pvtComment: self behaviorStrategy comment.
    newTrait behaviorStrategy pvtPackage: self behaviorStrategy package."

    originalName := self name.

    traitStrategy := RGTrait newStrategyFor: self environment.
    traitStrategy owner: self.
    traitStrategy pvtClassTrait: self pvtMetaclass.
    traitStrategy pvtPackage: self behaviorStrategy pvtPackage.
    traitStrategy pvtComment: self behaviorStrategy pvtComment.
    
    self behaviorStrategy: traitStrategy.

    self environment ask replaceName: originalName with: self name.
    
    
! !

!RGBehavior methodsFor:'copying'!

copyForBehaviorDefinitionPostCopy

    super copyForBehaviorDefinitionPostCopy.
    
    superclass := self superclass shallowCopy. "we need the superclass name"
    traitComposition := self traitComposition copyForBehaviorDefinition.
    tagsForMethods := self tagsForMethods copy asSet.
    tags := self tags copy asSet.
    
    behaviorStrategy := behaviorStrategy copyForBehaviorDefinitionWithOwner: self
! !

!RGBehavior methodsFor:'default model values'!

defaultLayout

    ^ self defaultFixedLayoutStubIn: self.
!

defaultLocalMethods

    ^ Set new
!

defaultTags

    ^ Set new
!

defaultTagsForMethods

    ^ Set new
!

defaultTraitComposition

    ^ self defaultTraitCompositionStubIn: self.
! !

!RGBehavior methodsFor:'documentation'!

extensions

    ^ self localMethods select: [ :each | each package ~= self package ]
! !

!RGBehavior methodsFor:'initialization'!

initialize

    super initialize.
    
    "use unresolved value for superclass to avoid infinite recursion. It needs to be set later"
    superclass := RGUnresolvedValue recursive.
    metaclass := RGUnresolvedValue recursive.
    localMethods := self unresolvedValue: self defaultLocalMethods.
    traitComposition := self unresolvedValue: self defaultTraitComposition.
    tagsForMethods := self unresolvedValue: self defaultTagsForMethods.	
    tags := self unresolvedValue: self defaultTags.	
!

initializeUnresolved

    super initializeUnresolved.

    superclass := self. "will be set later"
    metaclass := self. "will be set later"
    localMethods := self unresolvedValue: self defaultLocalMethods.
    traitComposition := self unresolvedValue: self defaultTraitComposition.
    tagsForMethods := self unresolvedValue: self defaultTagsForMethods.	
    tags := self unresolvedValue: self defaultTags.	
! !

!RGBehavior methodsFor:'lookup'!

lookupVar: aName
    ^ self layout 
        resolveSlot: aName asSymbol 
        ifFound: [:var | var] 
        ifNone: [self bindingOf: aName]
!

lookupVarForDeclaration: aName

    "This is a version of #lookupVar: that skips the OCRequestorScope, from here on it is the same as lookupVar:"

    ^ self lookupVar: aName
! !

!RGBehavior methodsFor:'managing container'!

addoptToParentStub

    self isRingResolved ifFalse: [ ^ self ].

    super addoptToParentStub.

    self parent pvtAddBehavior: self.
    

!

defaultParentStub

    ^ self defaultEnvironmentStub
! !

!RGBehavior methodsFor:'printing'!

printOn: aStream 
    aStream nextPutAll: self name
!

slotDefinitionString
    "Answer a string that contains an executable description of my Slots"

    "^self slots ifNotEmpty: [self slots asString] ifEmpty: ['{}']"
    
    ^String streamContents: [ :str | | special |
        str nextPutAll: '{ '.
        self slots do: [:slot |
                str nextPutAll: slot definitionString.
                special := slot needsFullDefinition]				
            separatedBy: [ 
                str nextPutAll: ' . '.  
                special ifTrue: [ str cr;tab;tab;tab;tab ]].
        str nextPutAll: ' }'. ]
! !

!RGBehavior methodsFor:'private'!

pvtLocalMethods: aCollection

    "use only for low-level copying"
    localMethods := aCollection
! !

!RGBehavior methodsFor:'private - backend access'!

pvtAddMethodTag: aSymbol
    
    tagsForMethods isRingResolved ifFalse: [ 
        self pvtCleanTagsForMethods  ].
    
    tagsForMethods add: aSymbol.	
    
!

pvtCleanTags

    tags := self defaultTags.
        
    "TODO:Announce if not empty"
    
    
!

pvtCleanTagsForMethods

    tagsForMethods := self defaultTagsForMethods.
        
    "TODO:Announce if not empty"
    
    
!

pvtMetaclass

    ^ metaclass value
!

pvtMetaclass: anRGMetaclass

    self environment verifyOwnership: anRGMetaclass.

    ^ metaclass := anRGMetaclass
!

pvtRemoveMethodTag: aSymbol
    
    tagsForMethods remove: aSymbol.
    
    "TODO:Announce"
    
    
!

pvtTagWith: aSymbol
    
    tags isRingResolved ifFalse: [ 
        self pvtCleanTags  ].
    
    tags add: aSymbol.	
    
!

pvtTags: aCollection

    ^ tags := aCollection
!

pvtTagsDo: aBlock

    ^ tags value do: aBlock
!

pvtTagsForMethodsDo: aBlock

    ^ tagsForMethods value do: aBlock
!

pvtUntagFrom: aSymbol
    
    (tags value includes: aSymbol)
        ifTrue: [ tags remove: aSymbol ].
    
    "TODO:Announce"
    
    
! !

!RGBehavior methodsFor:'private - backend interface'!

pvtAddLocalMethod: anRGMethod

    self environment verifyOwnership: anRGMethod.
    
    localMethods isRingResolved ifFalse: [ 
        self pvtCleanLocalMethods ].

    (self hasResolved: #tagsForMethods) 
        ifFalse: [ self pvtCleanTagsForMethods ].

    anRGMethod tags do: [ :aTag |
        self pvtAddMethodTag: aTag ].

    localMethods add: anRGMethod.	
    
!

pvtCleanLocalMethods

    localMethods := self defaultLocalMethods.
    
!

pvtLocalMethodsDo: aBlock

    ^ localMethods value do: aBlock
!

pvtRemoveLocalMethod: anRGMethod

    self environment verifyOwnership: anRGMethod.
    
    localMethods remove: anRGMethod.
    
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #superclass -> superclass. 
        #localMethods -> localMethods. 
        #traitComposition -> traitComposition.  
        #metaclass -> metaclass.  
        #tagsForMethods -> tagsForMethods. 
 		#tags -> tags. 
        }, self behaviorStrategy pvtResolvableProperties
    
    
!

pvtSuperclass

    ^ superclass value
    
!

pvtSuperclass: anRGBehavior

    anRGBehavior ifNotNil: [ 
        self environment verifyOwnership: anRGBehavior.].

    ^ superclass := anRGBehavior
    
!

pvtTraitComposition

    ^ traitComposition value
!

pvtTraitComposition: anRGTraitComposition

    self environment verifyOwnership: anRGTraitComposition.

    ^ traitComposition := anRGTraitComposition
! !

!RGBehavior methodsFor:'queries - methods'!

>> selector
    ^ self methodNamed: selector

    "Created: / 04-02-2021 / 08:56:16 / Jan Vrany <jan.vrany@labware.com>"
!

ensureLocalMethodNamed: aSymbol

    ^ self localMethodNamed: aSymbol ifAbsent: [ 
        | newMethod |
        newMethod := RGMethod named: aSymbol asSymbol parent: self.
        self addLocalMethod: newMethod.
        newMethod].
!

ensureUnresolvedLocalMethod

    | newMethod |
    newMethod := RGMethod parent: self.
    self addLocalMethod: newMethod.
    ^ newMethod
!

includesSelector: aString

    ^ self selectors includes: aString
!

localMethodNamed: aSymbol ifAbsent: aBlock

    self localMethodsDo: [ :each | (each name = aSymbol) ifTrue: [^ each]].
    
    ^ aBlock value.
!

localMethods

    ^ self localMethodsSet asArray
!

localMethodsSet

    | methods |
    
    methods := self defaultLocalMethods.
    self localMethodsDo: [ :each | methods add: each ].
    ^ methods
!

localSelectors

    ^ self localMethods collect: [ :each | each name ]
!

methodNamed: aSymbol

    | allMethods |
    allMethods := IdentitySet new.
    self methods do: [ :each | each name = aSymbol ifTrue: [^ each]].
    ^ nil
!

methods

    | methodsFromTraits methodsDict |
    "TODO: make nicer"
    "^ self propertyNamed: #methods ifAbsentPut: [ "
    methodsFromTraits := self traitComposition methods collect:  [ :each | each copy parent: self ].
    methodsDict := Dictionary new.
    methodsFromTraits do: [ :each |
        methodsDict at: each name put: each ].
    self localMethodsDo: [ :each |
        methodsDict at: each name put: each ].
    ^ methodsDict values asArray"]"
!

selectors

    ^ self methods collect: [ :each | each name ]
! !

!RGBehavior methodsFor:'queries - other'!

allInstVarNames
    "Answer an Array of the names of the receiver's instance variables. The 
    Array ordering is the order in which the variables are stored and 
    accessed by the interpreter."

    | vars |
    (self superclass == self or: [ self superclass isNil ])
        ifTrue: [vars := self instVarNames copy]	"Guarantee a copy is answered."
        ifFalse: [vars := self superclass allInstVarNames , self instVarNames].
    ^vars
! !

!RGBehavior methodsFor:'queries - protocols'!

ensureProtocolNamed: aSymbol
 
    ^ self ensureMethodTagNamed: aSymbol
!

protocols

    | methodTags |
    
    methodTags := self tagsForMethods.
    ^	methodTags
        ifEmpty: [ 
            self methods 
                ifEmpty: [ OrderedCollection new ]
                ifNotEmpty: [  OrderedCollection with: self class asYetUnclassifiedProtocolName   ].
            ]
        ifNotEmpty: [ methodTags  ]
! !

!RGBehavior methodsFor:'queries - tags'!

ensureMethodTagNamed: aSymbol

    ^ self tagsForMethods detect: [ :each | each asSymbol = aSymbol ] ifNone: [ 
        self addMethodTag: aSymbol.
        aSymbol ]
!

methodsTaggedWith: aSymbol

    ^ self localMethods select: [ :each | each isTaggedWith: aSymbol ]
! !

!RGBehavior methodsFor:'queries - testing'!

hasMethods
    "validates the existance of methods"

    ^ self methods notEmpty
!

hasTraitComposition

    ^	self traitComposition transformations isEmpty not
!

isAccessedIn: anRGMethod
    ^ (anRGMethod ast variableNodes select: [ :each | 
           each isGlobalVariable ]) anySatisfy: [ :each | 
          each name = self name ]
!

isRootInEnvironment		
        
    ^ self superclass == self or: [ self superclass isNil ] 
! !

!RGBehavior methodsFor:'resolving'!

makeResolved

    superclass := self superclass markAsRingResolved.
    localMethods := self localMethodsSet markAsRingResolved.
    traitComposition := self traitComposition markAsRingResolved.
    metaclass := self metaclass markAsRingResolved.
    tagsForMethods := self tagsForMethodsSet markAsRingResolved.
    tags := self tagsSet markAsRingResolved.
    
    super makeResolved.
! !

!RGBehavior methodsFor:'slots'!

slots		
        
    ^ self layout slots 
!

slotsNeedFullDefinition
    "return true if we define something else than InstanceVariableSlots"
    ^self slots anySatisfy: [ :each | each needsFullDefinition ]
! !

!RGBehavior methodsFor:'strategy'!

pvtAsTrait

    | traitStrategy |
    
    (self resolvedProperties includesAnyOf: #("superclass" traitComposition classVariables sharedPools))
        ifTrue: [self error: 'This class cannot be converted to trait'].
        
    "TODO: check empty layout"
    "newTrait := RGTrait unresolvedWithParent: self parent.
    newTrait copyPropertiesFrom: self.
    newTrait pvtName: name.
    newTrait pvtLocalMethods: localMethods.
    newTrait pvtTags: tags.

    newTrait behaviorStrategy pvtComment: self behaviorStrategy comment.
    newTrait behaviorStrategy pvtPackage: self behaviorStrategy package."

    traitStrategy := RGTrait newStrategyFor: self environment.

    traitStrategy pvtComment: self behaviorStrategy comment.
    traitStrategy pvtPackage: self behaviorStrategy package.
    
    self behaviorStrategy: traitStrategy.
    
    ^ self	
    
! !

!RGBehavior methodsFor:'testing'!

canMergeWith: anRGObject visited: visitedDefinitions

    | newVisited |

    (visitedDefinitions includes: self) ifTrue: [ ^ true ].
    
    (self isRingFullyResolved or: [ anRGObject isRingFullyResolved]) ifTrue: [^ true].

    (super canMergeWith: anRGObject visited: visitedDefinitions) ifFalse: [ ^ false ].

    newVisited := visitedDefinitions copyWith: self.
    
    self superclass ~~ self ifTrue: [ 
        (self superclass canMergeWith: anRGObject superclass visited: newVisited) ifFalse: [ ^ false ]].

    self metaclass ~~ self ifTrue: [ 
        (self metaclass canMergeWith: anRGObject metaclass visited: newVisited) ifFalse: [ ^ false ]].
    
    ^ true.
!

hasComment

    ^ self comment isEmptyOrNil not
!

hasResolvedSuperclass

    ^ self hasResolved: #superclass
!

isBehavior

    ^ true
!

isPointers

    ^ self isBits not
!

isTaggedWith: aSymbol

    ^self tags includes: aSymbol 
!

isWeak

    ^ self layout isWeakLayout
! !

!RGBehavior methodsFor:'testing - layouts'!

isBits

    ^ self layout isBitsLayout
!

isBytes

    ^ self layout isByteLayout 
!

isCompiledMethod

    ^ self layout isCompiledMethodLayout 
!

isEphemeron

    ^ self layout isEphemeronLayout
!

isEphemeronClass

    ^ self layout isEphemeronLayout 
!

isImmediateClass

    ^ self layout isImmediateLayout  
!

isVariable

    "is the definition a variable class?"

    ^ self layout isVariableLayout
!

isWords

    ^ self layout isWordLayout 
! !

!RGBehavior methodsFor:'testing class hierarchy'!

inheritsFrom: aClass 
    "Answer whether the argument, aClass, is on the receiver's superclass 
    chain."

    | aSuperclass |
    aSuperclass := self superclass.
    [aSuperclass == nil]
        whileFalse: 
            [aSuperclass == aClass ifTrue: [^true].
            aSuperclass := aSuperclass superclass].
    ^false
! !

!RGBehavior class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

