"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGLayout subclass:#RGPointerLayout
	instanceVariableNames:'slots'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGPointerLayout comment:''
!

!RGPointerLayout methodsFor:'accessing'!

layoutName

    ^ #PointerLayout
!

resolveSlot: aName ifFound: foundBlock ifNone: noneBlock

    self allSlots do: [ :slot | 
        slot name == aName ifTrue: [ ^ foundBlock cull: slot ] ].
    ^ noneBlock value
! !

!RGPointerLayout methodsFor:'accessing - backend'!

addSlot: anRGSlot 

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior addSlot: anRGSlot to: self ].
!

cleanSlots

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior cleanSlotsFor: self ].
    
    
!

removeSlot: anRGSlot

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior removeSlot: anRGSlot from: self ].
!

slotsDo: aBlock

    self backend forBehavior slotsFor: self do: aBlock
! !

!RGPointerLayout methodsFor:'default model values'!

defaultSlots

    ^ OrderedCollection new
! !

!RGPointerLayout methodsFor:'initialization'!

initialize

    super initialize.
    slots := self unresolvedValue: self defaultSlots.

!

initializeUnresolved

    super initializeUnresolved.
    slots := self unresolvedValue: self defaultSlots.

! !

!RGPointerLayout methodsFor:'private - backend access'!

pvtAddSlot: anRGSlot

    self environment verifyOwnership: anRGSlot.
    
    slots isRingResolved ifFalse: [ 
        self pvtCleanSlots  ].
    
    slots add: anRGSlot.	
!

pvtCleanSlots

    slots := self defaultSlots.
!

pvtRemoveSlot: anRGSlot

    self environment verifyOwnership: anRGSlot.
    
    slots remove: anRGSlot.
    
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #slots -> slots. 
    }
    
    
!

pvtSlotsDo: aBlock

    slots value do: aBlock
! !

!RGPointerLayout methodsFor:'queries - slots'!

allSlots 

    ^ self slots
!

instVarNames
    ^ self slots collect: [:each | each name]
!

slots

    | allSlots |
    allSlots := OrderedCollection new.
    self slotsDo: [ :each | allSlots add: each].
    ^ allSlots asArray
! !

!RGPointerLayout methodsFor:'resolving'!

makeResolved

    super makeResolved.
    
    slots := self slots asOrderedCollection markAsRingResolved.
! !

!RGPointerLayout methodsFor:'testing types'!

isPointerLayout

    ^ true
! !

