"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RBProgramNodeVisitor subclass:#RGTraitCompositionVisitor
	instanceVariableNames:'traitComposition'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Parsing'
!

RGTraitCompositionVisitor comment:'ast := (RBParser parseExpression: ''Trait1 + Trait2 + Trait3 + Trait4'').
ast := (RBParser parseExpression: ''(Trait2 - {#c})'').
ast := (RBParser parseExpression: ''Trait1 + (Trait2 - {#c})'').
ast := (RBParser parseExpression: ''Trait1 + (Trait2 - {#c}) + Trait3'').
ast := (RBParser parseExpression: ''Trait1 + (Trait2 - #(c ahoj bla: bla:bla:)) + Trait3'').

composition := RGTraitComposition unnamed.

visitor := RGTraitCompositionVisitor new.
visitor traitComposition: composition.
ast acceptVisitor: visitor.




'
!

!RGTraitCompositionVisitor class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGTraitCompositionVisitor methodsFor:'accessing'!

traitComposition
    ^ traitComposition
!

traitComposition: anObject
    traitComposition := anObject.
    traitComposition pvtCleanTransformations.
! !

!RGTraitCompositionVisitor methodsFor:'parsing'!

parse: anExpressionString for: anRGTraitCompositionDefinition

    | ast result |
    ast := (RBParser parseExpression: anExpressionString).
    self traitComposition: anRGTraitCompositionDefinition.
    result := ast acceptVisitor: self.
    
    result isCollection 
        ifFalse: [ result := OrderedCollection with: result. ].
        
    ^ result.
! !

!RGTraitCompositionVisitor methodsFor:'visiting'!

acceptArrayNode: anArrayNode 

    ^ anArrayNode children collect: [:each | self visitNode: each]

    "Created: / 23-12-2020 / 13:42:54 / Jan Vrany <jan.vrany@labware.com>"
!

acceptLiteralNode: anRBLiteralNode

    anRBLiteralNode value isArray ifTrue: [
        ^ anRBLiteralNode value collect: [:each | self visitNode: each]
    ] ifFalse: [
        ^ anRBLiteralNode value
    ].

    "Created: / 23-12-2020 / 13:45:02 / Jan Vrany <jan.vrany@labware.com>"
!

acceptMessageNode: aMessageNode 
    
    | rec args | 
    rec := (self visitNode: aMessageNode receiver).
    args := (aMessageNode arguments collect: [ :each | self visitNode: each ]).

    (aMessageNode selector = #+) ifTrue: [
        rec isCollection 
            ifTrue: [ rec add: args first ]
            ifFalse: [ rec := { rec. args first } asOrderedCollection ].
        ^ rec].
    
    (aMessageNode selector = #-) ifTrue: [ 
        | exclusion | 
        exclusion := RGTraitExclusion parent: self traitComposition.
        exclusion pvtSubject: rec.
        exclusion pvtCleanExclusions.
        args first do: [:each | exclusion pvtAddExclusion: each ]. 
        ^ exclusion].
    
    "an Array(Trait2 @ {#c3->#c2})"
    (aMessageNode selector = #@) ifTrue: [ 
        | exclusion | 
        exclusion := RGTraitAlias parent: self traitComposition.
        exclusion pvtSubject: rec.
        exclusion pvtCleanAliases.
        args first do: [:each | exclusion pvtAddAlias: each ]. 
        ^ exclusion].

    (aMessageNode selector = #->) ifTrue: [
        "for associations in aliases"
        ^ rec -> args first
        ].
    
    (aMessageNode selector = #classTrait) ifTrue: [
        "for associations in aliases"
        rec classTrait makeResolved.
        rec classTrait name: rec name, ' classTrait'.
        ^ rec classTrait.
        ].

    self error: 'Unknown trait composition message'.

    "Created: / 23-12-2020 / 13:45:26 / Jan Vrany <jan.vrany@labware.com>"
!

acceptSequenceNode: aSequenceNode 

    ^ aSequenceNode statements collect: [:each | self visitNode: each].

    "Created: / 23-12-2020 / 13:45:34 / Jan Vrany <jan.vrany@labware.com>"
!

acceptVariableNode: aVariableNode
    
    ^ self traitComposition environment ensureTraitNamed: aVariableNode name

    "Created: / 23-12-2020 / 13:45:40 / Jan Vrany <jan.vrany@labware.com>"
! !

!RGTraitCompositionVisitor class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

