"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGTraitDescriptionStrategy subclass:#RGTraitStrategy
	instanceVariableNames:'classTrait comment package'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGTraitStrategy comment:''
!

!RGTraitStrategy methodsFor:'accessing'!

makeResolved

    
    "try to set the correct name before resolving of it"
    ((self owner hasResolvedName not) and: [ self classTrait isRingResolved and: [ self classTrait hasResolvedName ] ]) ifTrue: [ 
        self owner pvtName: (self classTrait name withoutSuffix: ' classTrait') asSymbol.
         ].
    
    super makeResolved.
    classTrait := self classTrait markAsRingResolved.
    comment := self comment markAsRingResolved.
    package := self package markAsRingResolved.
    
!

package

    ^ self backend forBehavior traitPackageFor: self owner
!

package: anRGPackage

    self announceDefinitionChangeDuring: [ 
        self backend forBehavior setTraitPackageFor: self to: anRGPackage.
        self environment addPackage: anRGPackage.
        anRGPackage addDefinedBehavior: self owner. ].
!

pvtClassTrait

    ^ classTrait value
!

pvtClassTrait: anRGMetatraitDefinition

    self owner environment verifyOwnership: anRGMetatraitDefinition.

    ^ classTrait := anRGMetatraitDefinition
!

pvtComment

    ^ comment value
!

pvtComment: anRGComment

    self owner environment verifyOwnership: anRGComment.

    ^ comment := anRGComment
!

pvtPackage

    ^ package value
!

pvtPackage: anRGPackage

    self owner environment verifyOwnership: anRGPackage.

    ^ package := anRGPackage
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #classTrait -> classTrait. 
        #comment -> comment. 
        #package -> package. 
   	}
    
    
!

referencedBehaviors

    ^ super referencedBehaviors, {self classTrait}.
!

referencedPackages

    ^ Array with: package
!

sibling

    ^ self classTrait
!

theMetaClass
    ^ self classTrait
!

traitTransformationString

    ^ self owner name
! !

!RGTraitStrategy methodsFor:'private - backend access'!

acceptVisitor: aVisitor 

    ^ aVisitor visitTrait: self owner
!

baseTrait

    ^ self owner
!

category

    ^ self owner tags 
        ifEmpty: [ self owner package categoryName ]
        ifNotEmpty: [ 
            (self package hasResolvedName) 
                ifTrue: [ self owner package categoryName, '-', self owner tags first  ]
                ifFalse: [ self owner tags first  ]	] 
    
    "todo"
!

category: aString

    | aTag |
        
    self owner cleanTags.
    aTag := self package 
        ifNotNil: [ 
            self package name = aString
                ifTrue: [ ^ self ] "category contains only the package name"
                ifFalse: [ aString withoutPrefix: self package name, '-'  ]]
        ifNil: [ aString ].
    self owner tagWith: aTag asSymbol.

"	self backend forBehavior setCategoryFor: self to: aaString.
    
    self announcer behaviorDefinitionModified: self.
    self announcer behaviorRecategorized: self."
!

classTrait

    ^ self backend forBehavior classTraitFor: self owner
!

classTrait: anRGMetatraitDefinition

    self backend forBehavior setClassTraitFor: self owner to: anRGMetatraitDefinition
! !

!RGTraitStrategy methodsFor:'testing'!

classSide

    ^ self classTrait
!

comment

    ^ self backend forBehavior traitCommentFor: self owner
!

comment: anRGComment

    self backend forBehavior setTraitCommentFor: self owner to: anRGComment.
    
    self owner announcer behaviorCommentModified: self owner.
!

defaultCategory

    ^ nil
!

defaultComment

    ^ self owner defaultCommentStubIn: self owner
!

defaultPackage

    ^ self owner defaultPackageStubIn: self environment

    
!

definition
    "Answer a String that defines the receiver"

    ^String streamContents: [:stream |
        stream nextPutAll: 'Trait'.
        stream nextPutAll: ' named: ';
                store: self owner name.
        stream cr; tab; nextPutAll: 'uses: ';
                nextPutAll: self owner traitCompositionString.
        stream cr; tab; nextPutAll: 'package: ';
                store: self owner category asString].
!

hasComment

    ^ self comment isEmptyOrNil not
!

initialize

    super initialize.
    
    "classTrait is set in the superclass"
    comment := self unresolvedValue: self defaultComment.
    package := self unresolvedValue: self defaultPackage.
!

initializeUnresolved

    super initializeUnresolved.
    
    classTrait := RGUnresolvedValue recursive.
    comment := self unresolvedValue: self defaultComment.
    package := self unresolvedValue: self defaultPackage.
!

instanceSide
    ^ self baseTrait
!

isTraitStrategy

    ^ true
!

storeName

    ^ 'RGTrait'
! !

