"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGTraitV2DescriptionStrategy subclass:#RGTraitV2Strategy
	instanceVariableNames:'comment classVariables package sharedPools classTrait'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

!RGTraitV2Strategy class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGTraitV2Strategy methodsFor:'accessing - backend'!

package

    ^ self backend forBehavior packageFor: self owner
! !

!RGTraitV2Strategy methodsFor:'accessing - deprecated parallel hierarchy'!

theMetaClass

    ^ self owner metaclass
! !

!RGTraitV2Strategy methodsFor:'as yet unclassified'!

classTrait

    ^ self backend forBehavior classTraitFor: self owner
!

classTrait: anRGMetatraitDefinition

    self backend forBehavior setClassTraitFor: self owner to: anRGMetatraitDefinition
! !

!RGTraitV2Strategy methodsFor:'class variables'!

classVariableDefinitionString
    "Answer a string that evaluates to the definition of the class Variables"
    
    ^String streamContents: [ :str | | special |
        str nextPutAll: '{ '.
        self owner classVariables do: [:global |
                str nextPutAll: global definitionString.
                special := global needsFullDefinition]				
            separatedBy: [ 
                str nextPutAll: ' . '.  
                special ifTrue: [ str cr;tab;tab;tab;tab ]].
        str nextPutAll: ' }'. ]
! !

!RGTraitV2Strategy methodsFor:'default model values'!

defaultClassVariables

    ^ OrderedCollection new

    
!

defaultComment

    ^ self owner defaultCommentStubIn: self owner
!

defaultPackage

    ^ self owner defaultPackageStubIn: self environment

    
!

defaultSharedPools

    ^ OrderedCollection new

    
!

makeResolved

    
    "try to set the correct name before resolving of it"
    ((self owner hasResolvedName not) and: [ self owner metaclass isRingResolved and: [ self metaclass hasResolvedName ] ]) ifTrue: [ 
        self owner pvtName: (self owner metaclass name withoutSuffix: ' class') asSymbol.
         ].
    
    super makeResolved.
    
    classTrait := self classTrait markAsRingResolved.
    comment := self comment markAsRingResolved.
    classVariables := self classVariables markAsRingResolved.
    package := self package markAsRingResolved.
    sharedPools := self sharedPools markAsRingResolved.
! !

!RGTraitV2Strategy methodsFor:'enumerating'!

pvtSharedPoolsDo: aBlock

    sharedPools value do: aBlock
! !

!RGTraitV2Strategy methodsFor:'error handling'!

package: anRGPackage

    | oldPackage |
    oldPackage := self package.

    self owner announceDefinitionChangeDuring: [ 
        self owner backend forBehavior setPackageFor: self owner to: anRGPackage.
        self owner environment addPackage: anRGPackage.
        (oldPackage hasResolved: #definedBehaviors)
            ifTrue: [ oldPackage removeDefinedBehavior: self owner ].
        anRGPackage addDefinedBehavior: self owner. ].
    
    self owner announcer behaviorRepackaged: self owner from: oldPackage

    "Modified: / 22-12-2020 / 10:56:12 / Jan Vrany <jan.vrany@labware.com>"
! !

!RGTraitV2Strategy methodsFor:'initialization'!

initialize

    super initialize.
    
    comment := self unresolvedValue: self defaultComment.	
    classVariables := self unresolvedValue: self defaultClassVariables.	
    package := self unresolvedValue: self defaultPackage.	
    sharedPools := self unresolvedValue: self defaultSharedPools.	
            
    
!

initializeUnresolved

    super initializeUnresolved.
    
    comment := self unresolvedValue: self defaultComment.	
    classVariables := self unresolvedValue: self defaultClassVariables.	
    package := self unresolvedValue: self defaultPackage.	
    sharedPools := self unresolvedValue: self defaultSharedPools.	
    
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #comment -> comment. 
        #classVariables -> classVariables. 
        #package -> package. 
        #sharedPools -> sharedPools. 
    }
    
    
! !

!RGTraitV2Strategy methodsFor:'private - backend access'!

acceptVisitor: aVisitor 

    ^ aVisitor visitClass: self owner
!

addClassVariable: anRGInstanceVariableDefinition 

    self owner announceDefinitionChangeDuring: [ 
        self privAddClassVariable: anRGInstanceVariableDefinition ]
    
    
!

addSharedPool: anRGPoolVariable

    self owner announceDefinitionChangeDuring: [ 
        self privAddSharedPool: anRGPoolVariable ].
!

allClassVarNames

    ^self allClassVariables collect:[ :cvar| cvar name ]
!

allClassVariables
    "Answer a collection of the receiver's classVariables, including those defined its superclasses"

    ^ ((self owner superclass == nil) or: [ self owner superclass == self owner]) 
        ifTrue: [ self classVariables ]
        ifFalse: [ self owner superclass allClassVariables, classVariables ]
!

bindingOf: varName

    | aSymbol | 
    aSymbol := varName asSymbol.

    ^ (self innerBindingOf: aSymbol) ifNil: [
         self environment bindingOf: aSymbol ]
!

category

    ^ self owner tags 
        ifEmpty: [ self owner package categoryName ]
        ifNotEmpty: [ 
            (self owner package hasResolvedName) 
                ifTrue: [ self owner package categoryName, '-', self owner tags first  ]
                ifFalse: [ self owner tags first  ]	] 
    
    "todo"
!

category: aString

    | aTag |
        
    self owner cleanTagsWithoutAnnouncemnt.
    aTag := self owner package 
        ifNotNil: [ 
            self owner package name = aString
                ifTrue: [ ^ self owner ] "category contains only the package name"
                ifFalse: [ aString withoutPrefix: self package name, '-'  ]]
        ifNil: [ aString ].
    self owner tagWith: aTag asSymbol.

"	self backend forBehavior setCategoryFor: self to: aaString.
    
    self announcer behaviorDefinitionModified: self.
    self announcer behaviorRecategorized: self."
!

classSide

    ^ self owner metaclass
!

classVarNames

    ^ self classVariables collect: [ :each | each name ]
!

classVariables

    ^ Array streamContents: [ :str | self classVariablesDo: [ :each | str nextPut: each ] ]
!

classVariables: aCollectionOfSymbols

    self cleanClassVariables.
    aCollectionOfSymbols do: [ :classVarName |
        self addClassVariable: (RGClassVariable named: classVarName asSymbol parent: self).].
!

classVariablesDo: aBlock

    self backend forBehavior classVariablesFor: self owner do: aBlock
!

cleanClassVariables

    self owner announceDefinitionChangeDuring: [ 
        self backend forBehavior cleanClassVariablesFor: self owner ].
!

cleanSharedPools

    self backend forBehavior cleanSharedPoolsFor: self owner.
!

comment

    ^ self backend forBehavior commentFor: self owner
!

comment: anRGComment

    self backend forBehavior setCommentFor: self owner to: anRGComment.
    
    self owner announcer behaviorCommentModified: self.
!

copyForBehaviorDefinitionPostCopy

    | newVariables newSharedPools |
    
    self owner behaviorStrategy: self.
    
    super copyForBehaviorDefinitionPostCopy.
    
    newVariables := self classVariables collect: [ :each | each copyWithParent: self owner].
    newSharedPools := self sharedPools collect: [ :each | each shallowCopy. ].

    self pvtCleanClassVariables.
    self pvtCleanSharedPools.
    
    newVariables do: [ :each | self privAddClassVariable: each ].
    newSharedPools do: [ :each | self privAddSharedPool: each ].


!

innerBindingOf: aSymbol

    self classVariables detect: [ :each | each name = aSymbol  ] ifFound: [ :found | ^ found ].

    self sharedPoolsDo: [:pool | 
        ((self environment ask behaviorNamed: pool name) bindingOf: aSymbol) ifNotNil: [:binding | ^binding]].

    ((self owner superclass == self owner) or: [ self owner superclass isNil ]) ifFalse: [ ^ self owner superclass innerBindingOf: aSymbol].	
        
    ^ nil
!

privAddClassVariable: anRGInstanceVariableDefinition 

    self backend forBehavior addClassVariable: anRGInstanceVariableDefinition to: self owner	
    
!

privAddSharedPool: anRGPoolVariable
    
    self backend forBehavior addSharedPool: anRGPoolVariable to: self owner
!

pvtAddClassVariable: anRGInstanceVariableDefinition

    self owner environment verifyOwnership: anRGInstanceVariableDefinition.
    
    classVariables isRingResolved ifFalse: [ 
        self pvtCleanClassVariables  ].
    
    classVariables add: anRGInstanceVariableDefinition.	
!

pvtAddSharedPool: anRGPoolVariable

    self owner environment verifyOwnership: anRGPoolVariable.
    
    sharedPools isRingResolved ifFalse: [ 
        self pvtCleanSharedPools  ].
    
    sharedPools add: anRGPoolVariable.	
!

pvtClassTrait: anRGMetatraitDefinition

    self owner environment verifyOwnership: anRGMetatraitDefinition.

    ^ classTrait := anRGMetatraitDefinition
!

pvtClassVariablesDo: aBlock

    classVariables value do: aBlock
!

pvtCleanClassVariables

    classVariables := self defaultClassVariables.
!

pvtCleanSharedPools

    sharedPools := self defaultSharedPools.
!

pvtComment

    ^ comment value
!

pvtComment: anRGComment

    self owner environment verifyOwnership: anRGComment.

    ^ comment := anRGComment
!

pvtRemoveClassVariable: anRGInstanceVariableDefinition

    self owner environment verifyOwnership: anRGInstanceVariableDefinition.
    
    classVariables remove: anRGInstanceVariableDefinition.
!

pvtRemoveSharedPool: anRGPoolVariableDefinition

    self owner environment verifyOwnership: anRGPoolVariableDefinition.
    
    sharedPools remove: anRGPoolVariableDefinition.
!

removeClassVariable: anRGInstanceVariableDefinition

    self owner announceDefinitionChangeDuring: [ 
        self backend forBehavior removeClassVariable: anRGInstanceVariableDefinition from: self owner ].
!

removeSharedPool: anRGPoolVariable

    self owner announceDefinitionChangeDuring: [ 
        self backend forBehavior removeSharedPool: anRGPoolVariable from: self owner].
!

sharedPoolNames

    | allSharedPools |
    allSharedPools := OrderedCollection new.
    self sharedPoolsDo: [ :each | allSharedPools add: each name].
    ^ allSharedPools asArray
!

sharedPools

    | allSharedPools |
    allSharedPools := OrderedCollection new.
    self sharedPoolsDo: [ :each | allSharedPools add: (
        "use classes everywhare you can. TODO: really?"
        each isSymbol 
            ifTrue: [ self environment ask behaviorNamed: each name ]
            ifFalse: [ 
                (each isVariable and: [each isPoolVariable ])
                    ifTrue: [ 
                        | beh | 
                        beh := self environment ask behaviorNamed: each name.
                        beh ifNotNil: [ beh ] ifNil: [ each ] ]
                    ifFalse: [ each copy ]])].
    ^ allSharedPools 
!

sharedPools: aCollectionOfSymbols

    self cleanSharedPools.
    aCollectionOfSymbols do: [ :poolName |
        self addSharedPool: (RGPoolVariable named: poolName asSymbol parent: self).].
!

sharedPoolsDo: aBlock

    self owner backend forBehavior sharedPoolsFor: self owner do: aBlock
!

sibling

    ^ self owner metaclass.
! !

!RGTraitV2Strategy methodsFor:'testing'!

defaultMetaClass

    ^ self owner environment backend createNewUnresolvedMetaclassFor: self owner
!

definition
    "Answer a String that defines the receiver"

    ^String streamContents: [:stream |
        stream nextPutAll: 'Trait'.
        stream nextPutAll: ' named: ';
                store: self owner name.
        stream cr; tab; nextPutAll: 'uses: ';
                nextPutAll: self owner traitCompositionString.
        (self owner slots size > 0) ifTrue: [ 
            stream
                crtab;
                nextPutAll: 'slots: ';
                nextPutAll: self owner slotDefinitionString].
        stream cr; tab; nextPutAll: 'package: ';
                store: self category asString].
!

definitionWithSlots

    | aStream poolString|

    poolString := self owner sharedPoolsString.

    aStream := (String new: 800) writeStream.
    self owner superclass 
        ifNil: [aStream nextPutAll: 'ProtoObject']
        ifNotNil: [aStream nextPutAll: self owner superclass name].
    aStream nextPutAll: ' subclass: ';
            store: self owner name.
    (self owner hasTraitComposition) ifTrue: [
        aStream cr; tab; nextPutAll: 'uses: ';
            nextPutAll: self owner traitCompositionString].		
            
    (self layout layoutName = #FixedLayout) ifFalse: [
        aStream cr; tab; nextPutAll: 'layout: ';
            nextPutAll: self layout layoutName].
    aStream cr; tab; nextPutAll: 'slots: ';
            nextPutAll: self owner slotDefinitionString.
    aStream cr; tab; nextPutAll: 'classVariables: ';
            nextPutAll: self owner classVariableDefinitionString.
    poolString = '' ifFalse: [
        aStream cr; tab; nextPutAll: 'poolDictionaries: ';
            store: poolString].
    aStream cr; tab; nextPutAll: 'package: ';
            store: self category asString.

    self owner superclass ifNil: [ 
        aStream nextPutAll: '.'; cr.
        aStream nextPutAll: self owner name.
        aStream space; nextPutAll: 'superclass: nil'. ].

    ^ aStream contents
!

isClass

    ^ true
!

isClassStrategy

    ^ false
!

pvtClassTrait

    ^ classTrait value
!

pvtPackage

    ^ package value
!

pvtPackage: anRGPackage

    self owner environment verifyOwnership: anRGPackage.

    ^ package := anRGPackage
!

storeName

    ^ 'RGClass'
! !

