"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core/tests' }"

"{ NameSpace: Smalltalk }"

RGTest subclass:#RGCommentTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Tests-Core'
!

!RGCommentTest class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGCommentTest methodsFor:'tests'!

testClass

    | environment aClass aPackage aComment |
    
    environment := RGEnvironment new.
    aClass := environment ensureClassNamed: #SomeClass.
    aPackage := environment ensurePackageNamed: #SomePackage.
    aClass package: aPackage.
    aComment := RGComment parent: aClass.
    aComment content: 'some comment'.
    self deny: aClass hasComment.
    aClass comment: aComment.
    self assert: aComment package equals: aPackage.
    self assert: aClass hasComment.
    
    self assert: aComment fullName equals: 'SomeClass comment'.
!

testImplicitEnvironment
    
    | def def2 env |

    def := RGComment unnamed.
    env := def environment.

    self checkImplicitSingleClassEnvironmentOf: def parent.	

    self assert: (def isRingResolved).
    
    def2 := RGComment parent: def parent .
    self checkImplicitSingleClassEnvironmentOf: def parent.	
!

testMakeReslolved

    | comment |

    comment := RGComment unnamed.
    comment makeResolved.
    self assert: comment unresolvedProperties isEmpty.
    self assert: comment content equals: ''.
    self assert: comment author equals: ''.
    self assert: (comment time isKindOf: Timestamp).

    "Modified: / 22-12-2020 / 20:13:04 / Jan Vrany <jan.vrany@labware.com>"
!

testNewComment
    | comment time |
    
    comment := RGComment unnamed.
    
    self assert: (comment isRingResolved).
    self assert: (comment isComment).
    
    self assert: (comment hasUnresolvedAll: #(content author time name)).
    
    self assert: (comment isEmptyOrNil).
    
    self assert: comment content equals: ''.
    comment content: 'some class comment'.
    self assert: (comment hasUnresolvedAll: #(author time name)).
    self assert: comment content equals: 'some class comment'.
    self deny: (comment isEmptyOrNil).
    self assert: comment content equals: comment sourceCode.
    
    self assert: comment author equals: ''.
    comment author: 'FullAuthorName'.
    self assert: (comment hasUnresolvedAll: #(time name)).
    self assert: comment author equals: 'FullAuthorName'.
    
    time := Timestamp now.
    self assert: comment time equals: Timestamp epoch.
    comment time: time.
    self assert: (comment hasUnresolved: #name).
    self assert: comment time equals: time.
!

testResolvingConsistency

    | anObject |
    anObject := RGComment unnamed.
    self checkClassesConsistency: anObject and: anObject makeResolved
!

testTransformations
    | comment time |
    
    comment := RGComment unnamed.
    
    self assert: (comment isRingResolved).
    
    self assert: (comment hasUnresolvedAll: #(content author time name)).
    
    self assert: comment content equals: ''.
    comment content: 'some class comment'.
    self assert: (comment hasUnresolvedAll: #(author time name)).
    self assert: comment content equals: 'some class comment'.
    
    self assert: comment author equals: ''.
    comment author: 'FullAuthorName'.
    self assert: (comment hasUnresolvedAll: #(time name)).
    self assert: comment author equals: 'FullAuthorName'.
    
    time := Timestamp now.
    self assert: comment time equals: Timestamp epoch.
    comment time: time.
    self assert: (comment hasUnresolved: #name).
    self assert: comment time equals: time.
! !

!RGCommentTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

