"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#RGTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Tests-Core'
!

!RGTest class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGTest methodsFor:'assertions'!

assert: a equals: b
    self assert: a = b

    "Created: / 22-12-2020 / 13:08:28 / Jan Vrany <jan.vrany@labware.com>"
!

assert: a identicalTo: b
    self assert: a == b

    "Created: / 22-12-2020 / 12:17:19 / Jan Vrany <jan.vrany@labware.com>"
!

assertCollection: a hasSameElements: b
    self assert: a size = b size.
    a do: [:each | 
        self assert: (b includes: each)
    ].

    "Created: / 22-12-2020 / 13:15:14 / Jan Vrany <jan.vrany@labware.com>"
!

deny: a equals: b
    self deny: a = b

    "Created: / 22-12-2020 / 13:08:36 / Jan Vrany <jan.vrany@labware.com>"
!

deny: a identicalTo: b
    self deny: a == b

    "Created: / 22-12-2020 / 13:01:50 / Jan Vrany <jan.vrany@labware.com>"
! !

!RGTest methodsFor:'tests'!

checkClassesConsistency: object1 and: object2

    "check if instance variables have the same types (after sending of #value)"

    self assert: (object1 class instVarNames collect: [ :var | 
        var -> (object1 instVarNamed: var) value class ]) 
        equals: (object2 class instVarNames collect: [ :var | 
        var -> (object2 instVarNamed: var) value class ]) 
!

checkIncompatible: aBlock

    "prove that the block tries to do an action not suitable for a particular behavior strategy"

    self should: aBlock raise: RGIncompatibleBehaviorTypeError
! !

!RGTest methodsFor:'utilities'!

checkHierarchyConsistencyOf: anRGClass
    "prove that all relations between class, superclass, metaclass and superclass metaclass are consistent"

    self assert: anRGClass ~~ anRGClass superclass.
    self assert: anRGClass ~~ anRGClass metaclass.
    self assert: anRGClass ~~ anRGClass superclass metaclass.

    self assert: anRGClass superclass ~~ anRGClass metaclass.
    self assert: anRGClass superclass ~~ anRGClass superclass metaclass.
    self assert: anRGClass metaclass ~~ anRGClass superclass metaclass.

    self assert: anRGClass superclass superclass identicalTo: anRGClass superclass.
    self assert: anRGClass superclass metaclass identicalTo: anRGClass metaclass superclass.

    self assert: anRGClass superclass metaclass superclass identicalTo: anRGClass superclass metaclass.
    self assert: anRGClass superclass metaclass metaclass identicalTo: anRGClass superclass metaclass
!

checkImplicitSingleClassEnvironmentOf: anRGClass
    
    self deny: (anRGClass isRingResolved).

    "we have SomeClass, SomeClass class, SomeClass superclass, SomeClass superclass class"
    self assert: anRGClass environment ask behaviors size equals: 4.

    "we have one package for SomeClass and SomeClass class, other for SomeClass superclass 
    and SomeClass superclass class"	
    self assert: anRGClass environment ask packages size equals: 2.

    self checkHierarchyConsistencyOf: anRGClass.
    
    
    
!

checkImplicitSingleTraitEnvironmentOf: anRGTrait
    self assert: anRGTrait environment ask behaviors size equals: 10.
    self assert: anRGTrait environment ask packages size equals: 5.

    self checkHierarchyConsistencyOf: anRGTrait metaclass.

    self assert: anRGTrait ~~ anRGTrait classTrait.
    self assert: anRGTrait superclass identicalTo: anRGTrait superclass.
    self assert: anRGTrait classTrait superclass identicalTo: anRGTrait classTrait superclass
! !

!RGTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

